-- Prep module
local M = {}

-- Fire ring metrics
local fireRingCtrX = 528.918
local fireRingCtrY = -333.581
local fireRingCtrZ = 160.498
local fireRingRadius = 10
local fireRingCount = 40 -- Must be even and at least 4
--local fireRingCount = 10 -- Must be even and at least 4
local fireInterval = 0.100

-- Fire state triggers
local burningNot = 0
local burningSmolder = 1
local burningFire = 2
local burningExplosion = 3
local burningNoChange = 4
local selectedIgnition = burningExplosion

local cos = math.cos
local sin = math.sin
local pi = math.pi
local fireUnits = {}
local ringIgniting = 0
local fireIndex = 0
local fireTime = 0
local ugfxCalled = 0

----------------------
-- Reset everything --
----------------------
local function reset()
	fireUnits = {}
	ringIgniting = 0
	ugfxCalled = 0
	log("D", "reset", "Fire ring was reset")
end

--------------------------------
-- Start spawning a fire ring --
--------------------------------
-- Call this function via the following:
-- scripts_HighBeam9000__Fire_extension.spawnFireRing()
local function spawnFireRing()
	log("I", "spawnFireRing", "Spawning fire ring...")
	
	-- Spawn Fire
	local fireX
	local fireY
	local fireRef = nil
	local idxMax = fireRingCount - 1
	local fireAngle
	local angleOffset = pi * (7/4 - 1/40)
	for k = 0, idxMax do
		fireRef = core_vehicles.spawnNewVehicle("HighBeam9000_Fire", {config = "vehicles/HighBeam9000_Fire/RingFire.pc"})
		if fireRef == nil then break end
		fireUnits[k] = fireRef
		
		-- Set fire position
		fireAngle = 2 * pi * k / fireRingCount + angleOffset
		fireX = fireRingCtrX + fireRingRadius * cos(fireAngle)
		fireY = fireRingCtrY + fireRingRadius * sin(fireAngle)
		fireRef:setPosition(vec3(fireX, fireY, fireRingCtrZ))
	end
	
	-- Check for spawn failure
	if fireRef == nil then
		log("E", "spawnFireRing", "Cannot spawn fire ring!")
		reset()
		return
	end
	
	-- Done
	log("I", "spawnFireRing", "Fire ring spawned")
end

---------------------------------
-- Verify all fire units exist --
---------------------------------
local function verifyFire()
	local fireCount = 0
	for j, k in pairs(fireUnits) do
		fireCount = fireCount + 1
	end
	return fireCount
end

--------------------------------------
-- Ignite already-spawned fire ring --
--------------------------------------
-- Call this function via the following:
-- scripts_HighBeam9000__Fire_extension.igniteFireRing()
local function igniteFireRing()
	
	-- Turn on fire
	if verifyFire() == fireRingCount then
		ringIgniting = 1
		fireIndex = 0
		fireTime = 0
	end
end

------------------------------------------
-- Extinguish already-burning fire ring --
------------------------------------------
-- Call this function via the following:
-- scripts_HighBeam9000__Fire_extension.extinguishFireRing()
local function extinguishFireRing()
	
	-- Turn off fire
	ringIgniting = 0
	local luaCmd = "electrics.values['remoteTrig'] = " .. tostring(burningNot)
	for j, k in pairs(fireUnits) do
		k:queueLuaCommand(luaCmd)
	end
end

--------------------------------------
-- Delete already-spawned fire ring --
--------------------------------------
-- Call this function via the following:
-- scripts_HighBeam9000__Fire_extension.deleteFireRing()
local function deleteFireRing()
	
	-- Delete fires
	ringIgniting = 0
	for j, k in pairs(fireUnits) do
		k:delete()
	end
	fireUnits = {}
end

------------------------------------------
-- Extinguish already-burning fire ring --
------------------------------------------
-- Call this function via the following:
-- scripts_HighBeam9000__Fire_extension.extinguishFireRing()
local function extinguishFireRing()
	
	-- Turn off fire
	ringIgniting = 0
	local luaCmd = "electrics.values['remoteTrig'] = " .. tostring(burningNot)
	for j, k in pairs(fireUnits) do
		k:queueLuaCommand(luaCmd)
	end
end

------------------------------
-- Perform timed operations --
------------------------------
local function onUpdate(dtReal, dtSim, dtRaw)
	if ringIgniting == 1 then
		if fireTime >= fireInterval * fireIndex then
			local idxMax = (fireRingCount - 2) / 2 + 1
			if fireIndex <= idxMax then
				local luaCmd = "electrics.values['remoteTrig'] = " .. tostring(selectedIgnition)
				if fireIndex == 0 or fireIndex == idxMax then
					fireUnits[fireIndex]:queueLuaCommand(luaCmd)
--					log("I", "fireRing", "Igniting fire index " .. tostring(fireIndex))
				else
					fireUnits[fireIndex]:queueLuaCommand(luaCmd)
--					log("I", "fireRing", "Igniting fire index " .. tostring(fireIndex))
					fireUnits[fireRingCount - fireIndex]:queueLuaCommand(luaCmd)
--					log("I", "fireRing", "Igniting fire index " .. tostring(fireRingCount - fireIndex))
				end
				fireIndex = fireIndex + 1
			else
				ringIgniting = 0
			end
		end
		fireTime = fireTime + dtSim
	end
	
	if ugfxCalled == 0 then
		ugfxCalled = 1
		log("I", "fireRing", "onUpdate called")
	end
end

----------------------
-- Public interface --
----------------------
M.spawnFireRing = spawnFireRing
M.igniteFireRing = igniteFireRing
M.extinguishFireRing = extinguishFireRing
M.deleteFireRing = deleteFireRing
M.onExtensionLoaded = reset
M.onExtensionUnloaded = reset
M.reset = reset
M.onUpdate = onUpdate
return M
